# -*- coding: utf-8 -*-
"""
В данном файле хранятся фун. для интеграции behave с testlink.

Для выполнения тест плана необходимо у сценариев указать их id в testlink
в виде тегов (например esch-56, сценариям предназначем для создания зависимостей
которых нету в testlink нужно указать тег required)
и затем запустить behave c параметрами:
    PROJECT_NAME
    TESTPLAN_NAME
    BUILD_NAME
    Пример:
    behave --lang ru features/ -D PROJECT_NAME='PROJECT NAME'
        -D TESTPLAN_NAME='TESTPLAN NAME' -D BUILD_NAME="BUILD NAME"
"""

import testlink
from testlink.testlinkerrors import TLResponseError

# url api тестлинка
TESTLINK_API_URL = 'http://testms.bars-open.ru/lib/api/xmlrpc/v1/xmlrpc.php'

# ключ для подключения к api тестлинка
TESTLINK_API_KEY = '5b5ce143b5de8f6a2ed33d6e6bcb553b'

BEHAVE_TO_TESTLINK_STAUTS = {
    'passed': 'p',
    'failed': 'f',
    'skipped': 'b',
    'untested': 'n'
}

TESTLINK_PARAMS = {'PROJECT_NAME', 'TESTPLAN_NAME', 'BUILD_NAME'}


def setup_tsl_var(context, project_name, testplan_name, build_name):
    """
    Получаем инфо. о тестплане, id билда тест плана и список тест кейсов
    для выполнения.
    """
    context.tsl = testlink.TestlinkAPIClient(
        TESTLINK_API_URL,
        TESTLINK_API_KEY
    )
    try:
        context.testplan_info = context.tsl.getTestPlanByName(
            project_name,
            testplan_name
        )[0]
    except TLResponseError as tl_err:
        print tl_err.code

    testplan_builds = context.tsl.getBuildsForTestPlan(
        context.testplan_info['id']
    )
    for build in testplan_builds:
        if build['name'] == build_name:
            context.testplan_buildid = build['id']
            break

    testcases_info = context.tsl.getTestCasesForTestPlan(
        context.testplan_info['id']
    )
    context.testcases_id = []
    for key, value in testcases_info.items():
        context.testcases_id.append(value[0]['full_external_id'])


def send_report_to_tsl(context, scenario):
    """
    Формируем сообщение о прохождение сценария и отправляем в тестлинк
    """
    external_id = (set(scenario.tags) & set(context.testcases_id)).pop()
    msg = 'Scenario passed.'
    if scenario.status == 'failed':
        msg = u'Scenario failed at step "{}".'.format(
            context.failed_step
        )
    elif scenario.status == 'skipped':
        msg = 'Scenario skipped.'

    context.tsl.reportTCResult(
        testplanid=context.testplan_info['id'],
        status=BEHAVE_TO_TESTLINK_STAUTS[scenario.status],
        testcaseexternalid=str(external_id),
        buildid=context.testplan_buildid,
        notes=msg
    )


def testlink_setup(function):
    """
    Декоратор для фун. before_all из файла environment,
    для инициализации testlinkapi.
    """
    def wrapper_testlink_setup(arg):
        if TESTLINK_PARAMS.issubset(arg.config.userdata):
            setup_tsl_var(arg,
                          project_name=arg.config.userdata['PROJECT_NAME'],
                          testplan_name=arg.config.userdata['TESTPLAN_NAME'],
                          build_name=arg.config.userdata['BUILD_NAME']
                          )
            arg.execute_all = False
        else:
            arg.execute_all = True
            arg.testcases_id = ()
        function(arg)
    return wrapper_testlink_setup


def scenario_execute(function):
    """
    Декоратор для фун. before_scenario, на основе тест плана определяет
    выполнять сценарий или нет.
    """
    def wrapper_scenario_execute(arg_context, arg_scenario):
        scenario_in_testplan = (
            set(arg_scenario.tags) & set(arg_context.testcases_id)
            or 'required' in arg_scenario.tags)
        if not scenario_in_testplan and not arg_context.execute_all:
            arg_scenario.skip()
        function(arg_context, arg_scenario)
    return wrapper_scenario_execute


def send_report(function):
    """
    Декоратор для фун. after_scenario, если сценарий в тестплане то отправляет
    инф. о его выполнении в testlink
    """
    def wrapper_send_report(arg_context, arg_scenario):
        if set(arg_scenario.tags) & set(arg_context.testcases_id) \
                and not arg_context.execute_all:
            send_report_to_tsl(arg_context, arg_scenario)
        function(arg_context, arg_scenario)
    return wrapper_send_report
